/*
** Copyright (C) 2006 Thadeu Lima de Souza Cascardo <cascardo@minaslivre.org>
** Copyright (C) 2009 Thadeu Lima de Souza Cascardo <cascardo@holoscopio.com>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/

#include <gnet.h>
#include <glib.h>
#include <string.h>
#include "nethook.h"
#include "pop.h"
#include "usermap.h"

typedef struct
{
  net_read orig_read;
  gpointer orig_data;
  GString *buffer;
  GString *line;
  gchar *user;
} pop_t;

static int
pop_check_user (pop_t *pop)
{
  gchar *end;
  gchar *s;
  end = pop->line->str + pop->line->len;
  s = pop->line->str;
  while (s < end && *s == ' ') s++;
  if (end - s < 5)
    return -1;
  if (g_ascii_strncasecmp (s, "USER ", 5) != 0)
    return -1;
  s += 5;
  while (s < end && *s == ' ') s++;
  if (s == end)
    return -1;
  end--;
  while (end >= s && (*end == '\n' || *end == '\r')) end--;
  if (end < s)
    return -1;
  if (pop->user)
    g_free (pop->user);
  pop->user = g_strndup (s, end - s + 2);
  pop->user[end - s + 1] = 0;
  return 0;
}

static int
pop_getline (pop_t *pop)
{
  char * end;
  size_t len;
  if (pop->buffer->len == 0)
    return -1;
  end = memchr (pop->buffer->str, '\n', pop->buffer->len);
  if (end == NULL)
    return -1;
  len = end - pop->buffer->str + 1;
  g_string_truncate (pop->line, 0);
  g_string_append_len (pop->line, pop->buffer->str, len);
  g_string_erase (pop->buffer, 0, len);
  return 0;
}

static void
pop_read (net_hook_t *hook, gchar *buffer, size_t len)
{
  pop_t *pop = hook->data;
  g_string_append_len (pop->buffer, buffer, len);
  while (pop_getline (pop) == 0)
    {
      if (pop_check_user (pop) == 0)
        {
          g_message ("User is trying to authenticate as %s.", pop->user);
          if (usermap_perm (pop->user) == ACCESS_DENY)
            {
              g_message ("Denying access to user %s.", pop->user);
              pop_destroy (hook);
              gnet_conn_disconnect (hook->conn);
              return;
            }
        }
      hook->data = pop->orig_data;
      pop->orig_read (hook, pop->line->str, pop->line->len);
      hook->data = pop;
    }
}

net_hook_t *
pop_hook_new (net_hook_t *layer)
{
  pop_t *pop;
  pop = g_slice_new (pop_t);
  pop->orig_read = layer->read;
  pop->orig_data = layer->data;
  pop->buffer = g_string_sized_new (4096);
  pop->line = g_string_sized_new (4096);
  pop->user = NULL;
  layer->read = pop_read;
  layer->data = pop;
  return layer;
}

void
pop_destroy (net_hook_t *hook)
{
  pop_t *pop = hook->data;
  g_string_free (pop->buffer, TRUE);
  g_string_free (pop->line, TRUE);
  if (pop->user)
    g_free (pop->user);
  hook->read = pop->orig_read;
  hook->data = pop->orig_data;
  g_slice_free (net_hook_t, (gpointer) pop);
}
